﻿#include "CDobotConnector.h"
#include <QSerialPort>
#include <QSerialPortInfo>
#include "DobotType.h"
#include "DobotDevice/UdpSearch.h"
#include "DobotDevice/UdpPort.h"
#include <QThread>
#include <QCoreApplication>
//#include "HID/hidapi.h"

CDobotConnector::CDobotConnector()
    : QObject(0)
    , ioDevice(NULL)
//    , hidDevice(NULL)
//    , hidIndex(1)
//    , hidMap()
{
//    hid_init();
}

CDobotConnector::~CDobotConnector()
{
//    hid_exit();
}

void CDobotConnector::onInit(void)
{
    //qDebug() << metaObject()->className() << ":" << QThread::currentThread();
}

//Q_SIGNAL void newConnectStatus(bool connected, QIODevice *ioDevice);

void CDobotConnector::onBytesToWrite(const char *data, qint64 maxSize)
{
    if (ioDevice) {
        ioDevice->write(data, maxSize);
    }
//    else if(hidDevice) {
//        uint8_t buffer[65];
//        int index = 0;
//        int count = maxSize;
//        buffer[0] = 0;
//        while (count) {
//            if (count > 63) {
//                buffer[1] = 63;
//                memcpy(&buffer[2], &data[index], 63);
//                hid_write(hidDevice, buffer, 65);
//                index += 63;
//                count -= 63;
//            } else {
//                buffer[1] = count;
//                memcpy(&buffer[2], &data[index], 63);
//                hid_write(hidDevice, buffer, 65);
//                index += count;
//                count -= count;
//            }

//            emit bytesWritten();
//        }
//    }
}

//Q_SIGNAL void bytesWritten(void);

void CDobotConnector::onBytesReady(void)
{
    emit bytesReady(ioDevice->readAll());
}

//Q_SIGNAL void bytesReady(QByteArray data);
void CDobotConnector::searchDobot(void *isFinishedAddr, void *resultAddr, void *dobotNameListAddr, unsigned int maxLen)
{
    bool *isFinished = (bool *)isFinishedAddr;
    int *result = (int *)resultAddr;

    char *dobotNameList = (char *)dobotNameListAddr;

    QList<QSerialPortInfo> availablePorts = QSerialPortInfo::availablePorts();
    QStringList udpPortList;
    UdpSearch::Instance(this)->SearchDobotDevice(&udpPortList);
    dobotPortNameList.clear();

    foreach(QSerialPortInfo portInfo, availablePorts) {
        if (portInfo.description().contains("USB-to-Serial")
            || portInfo.description().contains("CH340")
            || portInfo.description().contains("CP210")
            || portInfo.description().contains("USB2.0-Serial")
            || portInfo.description().contains("USB Serial Port")) {
            dobotPortNameList.push_back(portInfo.portName());
        }
    }

//    // Update all the matched hid device
//    hid_device_info *hidDeviceInfo = hid_enumerate(HID_VID,HID_PID);
//    hid_device_info *nextDeviceInfo = hidDeviceInfo;
//    while (nextDeviceInfo) {
//        QString hidIndexStr;

//        if (hidMap.values().contains(nextDeviceInfo->path) == false) {
//            hidIndexStr = QString("HID%1").arg(hidIndex);
//            hidIndex++;
//            hidMap.insert(hidIndexStr, nextDeviceInfo->path);
//        } else {
//            hidIndexStr = hidMap.key(nextDeviceInfo->path);
//        }

//        dobotPortNameList.push_back(hidIndexStr);
//        nextDeviceInfo = nextDeviceInfo->next;
//    }
//    hid_free_enumeration(hidDeviceInfo);

    // Now all the dobot informations are stored in dobotPortNameList
    do {
        if (dobotNameList == NULL) {
            break;
        }
        strcpy(dobotNameList, "");
        foreach(const QString &dobotStr, dobotPortNameList) {
            std::string stdString = dobotStr.toStdString();
            const char *portName = stdString.c_str();
            uint32_t nextLen = strlen(dobotNameList) + 1/*\0*/ + strlen(portName) + 1/* */;

            if (nextLen > maxLen) {
                break;
            }
            strcat(dobotNameList, portName);
            strcat(dobotNameList, " ");
        }

        for (int i = 0; i < udpPortList.size(); ++i){
            const char *idAddr = udpPortList.at(i).toLocal8Bit().constData();
            uint32_t nextLen = strlen(dobotNameList) + 1/*\0*/ + strlen(idAddr) + 1/* */;

            if (nextLen > maxLen) {
                break;
            }
            strcat(dobotNameList, idAddr);
            strcat(dobotNameList, " ");
        }

        // Trim the last blank
        if (dobotNameList[strlen(dobotNameList) - 1] == ' ') {
            dobotNameList[strlen(dobotNameList) - 1] = '\0';
        }
    } while (0);

    *result = dobotPortNameList.count() + udpPortList.count();
    *isFinished = true;
    return;
}

void CDobotConnector::connectDobot(void *isFinishedAddr, void *resultAddr, void *portNameAddr, unsigned int baudrate, void *fwTypeAddr, void *versionAddr)
{
    bool *isFinished = (bool *)isFinishedAddr;
    int *result = (int *)resultAddr;
    char *fwType = (char *)fwTypeAddr;
    char *version = (char *)versionAddr;

    const char *portName = (const char *)portNameAddr;
    QString portNameStr(portName);

    if (portNameStr.contains("HID") == false) {
        QString connectPortName;
        if (portName == NULL || strcmp(portName, "") == 0) {
            bool _isFinished;
            int _result;
            searchDobot((void *)&_isFinished, (void *)&_result, 0, 0);
            if (dobotPortNameList.count() < 1) {
                *result = DobotConnect_NotFound;
                *isFinished = true;
                return;
            }
            connectPortName = dobotPortNameList.at(0);
        } else{
            connectPortName = QString(portName);
        }

        QRegExp regExpIP("((25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])[\\.]){3}(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])");
        if (!regExpIP.exactMatch(connectPortName)) {
            QSerialPort *serialPort = new QSerialPort(connectPortName, this);
            serialPort->setBaudRate(baudrate);
            serialPort->setDataBits(QSerialPort::Data8);
            serialPort->setParity(QSerialPort::NoParity);
            serialPort->setStopBits(QSerialPort::OneStop);
            ioDevice = serialPort;
        } else {
            if (fwType) {
                strcpy(fwType, "DobotWifi");
            }
            if (version) {
                strcpy(version, "0.0.0");
            }

            UdpPort *udpPort = new UdpPort(this);
            udpPort->setAddr(connectPortName);
            ioDevice = udpPort;
        }
        if (ioDevice->open(QIODevice::ReadWrite) == false) {
            ioDevice->deleteLater();
            ioDevice = 0;
            // Serial port is occupied!
            *result = DobotConnect_Occupied;
            *isFinished = true;
            return;
        }

        /****************************
         *
         * Check Room Type
         *
         ****************************/
        if (fwType && strcmp(fwType, "DobotWifi") != 0) {
            QRegExp regExpGRBL("GRBL\\:\\sV((\\d+\\.){2}\\d+)");
            QRegExp regExpMARLIN("MARLIN\\:\\sV((\\d+\\.){2}\\d+)");

            const uint32_t recvSize = 100;
            char recvData[recvSize];
            QString recvStr;

            ioDevice->write("\nM10\nM10\nM10\nM10\nM10\n");
            //ioDevice->flush();

            //Recv and check
            QDateTime startCheckTime(QDateTime::currentDateTime());
            QDateTime currentTime(startCheckTime);
            while(true) {
                if(startCheckTime.msecsTo(currentTime) > 500) {
                    if (fwType) {
                        strcpy(fwType, "DobotSerial");
                    }
                    if (version) {
                        strcpy(version, "0.0.0");
                    }
                    break;
                }

                memset(recvData, 0, recvSize);
                if(ioDevice->read(recvData, recvSize) > 0) {
                    recvStr += QString(recvData);

                    if(recvStr.indexOf(regExpGRBL) >= 0) {
                        printf("GRBL\n");
                        if (fwType) {
                            strcpy(fwType, "GRBL");
                        }
                        if (version) {
                            strcpy(version, (const char*)regExpGRBL.cap(1).toLatin1().data());
                        }
                        ioDevice->close();
                        ioDevice->deleteLater();
                        ioDevice = NULL;
                        break;
                    }

                    if(recvStr.indexOf(regExpMARLIN) >= 0) {
                        printf("MARLIN\n");
                        if (fwType) {
                            strcpy(fwType, "MARLIN");
                        }
                        if (version) {
                            strcpy(version, (const char*)regExpMARLIN.cap(1).toLatin1().data());
                        }
                        ioDevice->close();
                        ioDevice->deleteLater();
                        ioDevice = NULL;
                        break;
                    }
                }
                QCoreApplication::processEvents(QEventLoop::AllEvents, 10);
                currentTime = QDateTime::currentDateTime();
            }
        }
    }
//    else{
//        // Open hid devce
//        if (hidDevice) {
//            QTimer *timer = findChild<QTimer *>("hidReadTimer");
//            if (timer) {
//                timer->stop();
//                timer->deleteLater();
//            }
//            hid_close(hidDevice);
//            hidDevice = 0;
//        }
//        hidDevice = hid_open_path(hidMap.value(portNameStr).toStdString().c_str());
//        if (!hidDevice) {
//            *result = DobotConnect_NotFound;
//            *isFinished = true;
//            return;
//        }
//        hid_set_nonblocking(hidDevice, true);
//        QTimer *timer = new QTimer(this);
//        timer->setObjectName("hidReadTimer");
//        connect(timer, &QTimer::timeout, [=](){
//            unsigned char buffer[64];
//            int len = hid_read(hidDevice, buffer, sizeof(buffer));
//            if (len) {
//                len = buffer[0];
//                QByteArray data;
//                data.resize(len);
//                memcpy(data.data(), &buffer[1], len);
//                emit bytesReady(data);
//            }
//        });
//        timer->start(1);
//    }

    if (ioDevice) {
        connect(ioDevice, SIGNAL(bytesWritten(qint64)), this, SIGNAL(bytesWritten()));
        connect(ioDevice, SIGNAL(readyRead()), this, SLOT(onBytesReady()));

        emit newConnectStatus(true);
    }
//    else if(hidDevice) {
//        emit newConnectStatus(true);
//    }
    *result = DobotConnect_NoError;
    *isFinished = true;
    return;
}

void CDobotConnector::disconnectDobot(void *isFinishedAddr, void *resultAddr)
{
    bool *isFinished = (bool *)isFinishedAddr;
    int *result = (int *)resultAddr;

    emit newConnectStatus(false);
    if (ioDevice) {
        ioDevice->close();
        ioDevice->deleteLater();
        ioDevice = 0;
    }
//    else if (hidDevice) {
//        QTimer *timer = findChild<QTimer *>("hidReadTimer");
//        if (timer) {
//            timer->stop();
//            timer->deleteLater();
//        }
//        hid_close(hidDevice);
//        hidDevice = 0;
//    }

    *result = DobotConnect_NoError;
    *isFinished = true;
    return;
}
